"""Main class to test working of LG Horizon API"""

import asyncio
import json
import logging
import sys  # Import sys for stdin

import aiohttp

from lghorizon.lghorizon_api import LGHorizonApi
from lghorizon.lghorizon_models import LGHorizonAuth

# Define an asyncio Event to signal shutdown
shutdown_event = asyncio.Event()


async def read_input_and_signal_shutdown():
    """Reads a line from stdin and sets the shutdown event."""
    _LOGGER.info("Press Enter to gracefully shut down...")
    # run_in_executor is used to run blocking I/O operations in a separate thread
    # so it doesn't block the asyncio event loop.
    await asyncio.get_event_loop().run_in_executor(None, sys.stdin.readline)
    _LOGGER.info("Enter pressed, signaling shutdown.")
    shutdown_event.set()


_LOGGER = logging.getLogger(__name__)


async def main():
    """main loop"""
    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        filename="lghorizon.log",
        filemode="w",
    )

    logging.info("Starting LG Horizon test script")
    with open("secrets.json", encoding="utf-8") as f:
        secrets = json.load(f)
        username = secrets.get("username")
        password = secrets.get("password")
        country = secrets.get("country", "nl")

    async with aiohttp.ClientSession() as session:
        auth = LGHorizonAuth(session, country, username=username, password=password)
        api = LGHorizonApi(auth)

        # Start the input reader task
        input_task = asyncio.create_task(read_input_and_signal_shutdown())

        async def device_callback(device_id: str):
            device = devices[device_id]
            print(
                f"Device {device.device_id} state changed. Status:\n\nName: {device.device_friendly_name}\nState: {device.device_state.state.value}\nChannel: {device.device_state.channel_name}\nTitle: {device.device_state.title}\nSubtitle: {device.device_state.sub_title}\nSource type: {device.device_state.source_type.value}\n\n",
            )

        try:
            await api.initialize()
            devices = await api.get_devices()
            for device in devices.values():
                await device.set_callback(device_callback)
            quota = await api.get_recording_quota()
            print(f"Recording occupancy: {quota.percentage_used}")
            try:
                recordings = await api.get_all_recordings()
                print(f"Total recordings: {recordings.total}")

                show_recordings = await api.get_show_recordings(
                    "crid:~~2F~~2Fbds.tv~~2F272418335", "NL_000006_019130"
                )
                print(f"recordings: {show_recordings.total}")
            except Exception as ex:
                print(ex)

            # Wait until the shutdown event is set
            await shutdown_event.wait()

        except Exception as e:
            print(f"An error occurred: {e}")
            _LOGGER.error("An error occurred: %s", e, exc_info=True)
        finally:
            _LOGGER.info("Shutting down API and cancelling input task.")
            input_task.cancel()
            try:
                await input_task  # Await to let it clean up if it was cancelled
            except asyncio.CancelledError:
                pass  # Expected if cancelled
            await api.disconnect()
            _LOGGER.info("Shutdown complete.")


asyncio.run(main())



import asyncio
import json
import logging

import paho.mqtt.client as mqtt
from typing import Any, Callable, Coroutine
from .helpers import make_id
from .lghorizon_models import LGHorizonAuth

_logger = logging.getLogger(__name__)


class LGHorizonMqttClient:
    """LGHorizon MQTT client."""

    _mqtt_broker_url: str = ""
    _mqtt_client: mqtt.Client
    _auth: LGHorizonAuth
    _mqtt_token: str = ""
    client_id: str = ""
    _on_connected_callback: Callable[[], Coroutine[Any, Any, Any]]
    _on_message_callback: Callable[[dict, str], Coroutine[Any, Any, Any]]

    @property
    def is_connected(self):
        """Is client connected."""
        return self._mqtt_client.is_connected

    def __init__(
        self,
        auth: LGHorizonAuth,
        on_connected_callback: Callable[[], Coroutine[Any, Any, Any]],
        on_message_callback: Callable[[dict, str], Coroutine[Any, Any, Any]],
    ):
        """Initialize the MQTT client."""
        self._auth = auth
        self._on_connected_callback = on_connected_callback
        self._on_message_callback = on_message_callback
        self._loop = asyncio.get_event_loop()

    @classmethod
    async def create(
        cls,
        auth: LGHorizonAuth,
        on_connected_callback: Callable[[], Coroutine[Any, Any, Any]],
        on_message_callback: Callable[[dict, str], Coroutine[Any, Any, Any]],
    ):
        """Create the MQTT client."""
        instance = cls(auth, on_connected_callback, on_message_callback)
        service_config = await auth.get_service_config()
        mqtt_broker_url = await service_config.get_service_url("mqttBroker")
        instance._mqtt_broker_url = mqtt_broker_url.replace("wss://", "").replace(
            ":443/mqtt", ""
        )
        instance.client_id = await make_id()
        instance._mqtt_client = mqtt.Client(
            client_id=instance.client_id,
            transport="websockets",
        )

        instance._mqtt_client.ws_set_options(
            headers={"Sec-WebSocket-Protocol": "mqtt, mqttv3.1, mqttv3.11"}
        )
        instance._mqtt_token = await auth.get_mqtt_token()
        instance._mqtt_client.username_pw_set(auth.household_id, instance._mqtt_token)
        instance._mqtt_client.tls_set()
        instance._mqtt_client.enable_logger(_logger)
        instance._mqtt_client.on_connect = instance._on_connect
        instance._on_connected_callback = on_connected_callback
        instance._on_message_callback = on_message_callback
        return instance

    def _on_connect(self, client, userdata, flags, result_code):  # pylint: disable=unused-argument
        if result_code == 0:
            self._mqtt_client.on_message = self._on_message
            if self._on_connected_callback:
                asyncio.run_coroutine_threadsafe(
                    self._on_connected_callback(), self._loop
                )
        elif result_code == 5:
            self._mqtt_client.username_pw_set(self._auth.household_id, self._mqtt_token)
            asyncio.run_coroutine_threadsafe(self.connect(), self._loop)
        else:
            _logger.error(
                "Cannot connect to MQTT server with resultCode: %s", result_code
            )

    def _on_message(self, client, userdata, message):  # pylint: disable=unused-argument
        """Wrapper for handling MQTT messages in a thread-safe manner."""
        asyncio.run_coroutine_threadsafe(
            self._on_client_message(client, userdata, message), self._loop
        )

    async def connect(self) -> None:
        """Connect the client."""
        self._mqtt_client.connect(self._mqtt_broker_url, 443)
        self._mqtt_client.loop_start()

    async def subscribe(self, topic: str) -> None:
        """Subscribe to a MQTT topic."""
        self._mqtt_client.subscribe(topic)

    async def publish_message(self, topic: str, json_payload: str) -> None:
        """Publish a MQTT message."""
        self._mqtt_client.publish(topic, json_payload, qos=2)

    async def disconnect(self) -> None:
        """Disconnect the client."""
        if self._mqtt_client.is_connected():
            self._mqtt_client.disconnect()

    async def _on_client_message(self, client, userdata, message):  # pylint: disable=unused-argument
        """Handle messages received by mqtt client."""
        json_payload = await self._loop.run_in_executor(
            None, json.loads, message.payload
        )
        _logger.debug(
            "Received MQTT message \n\ntopic: %s\npayload:\n\n%s\n",
            message.topic,
            json.dumps(json_payload, indent=2),
        )
        if self._on_message_callback:
            await self._on_message_callback(json_payload, message.topic)
